/* TTMPPlat.h
 *
 * Platform dependent TTMP declarations for Linux.
 *
 * Copyright (C) 2005-2006 TomTom BV <http://www.tomtom.com/>
 * Author 0: Dimitry Andric <dimitry.andric@tomtom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2
 * as published by the Free Software Foundation.
 */

#ifndef INCLUDED_TTMPPLAT_H
#define INCLUDED_TTMPPLAT_H

#include <sys/types.h>   /* for socket(2) */
#include <sys/socket.h>  /* for socket(2) */
#include <sys/un.h>      /* for struct sockaddr_un */
#include <netinet/in.h>  /* for struct sockaddr_in */
#include <errno.h>       /* for errno */
#include <unistd.h>      /* for close(2) */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/* Debug message macros */
#ifdef DEBUG
#define TTMPDBG(fmt, arg...) do { fprintf(stderr, "%s: " fmt, __FUNCTION__ ,##arg); } while (0)
#else
#define TTMPDBG(fmt, arg...) do {} while (0)
#endif
#define TTMPMSG(fmt, arg...) do { fprintf(stderr, "%s: " fmt, __FUNCTION__ ,##arg); } while (0)
#define TTMPERR(fmt, arg...) do { fprintf(stderr, "%s: INTERNAL ERROR: " fmt, __FUNCTION__ ,##arg); } while (0)

/* Attribute for functions that do not return */
#define NORETURN __attribute__((noreturn))

/* longlong: ISO C long long type (usually 64 bits) */
typedef long long longlong;

/* handling 64 bits literals */
#define LONGLONG(x) x##LL

/* 64 bits scanf prefix */
#define SCANF_LL "L"

/* 64 bits printf prefix */
#define PRINTF_LL "ll"

/* size_t printf prefix */
#define PRINTF_SZ "z"

/* ssize_t: ISO C signed size_t type */
/* ssize_t defined in sys/types.h */

/* TTMPSocket: socket handle type */
typedef int TTMPSocket;

/* TTMPSockAddr: socket address union */
typedef union uTTMPSockAddr {
	struct sockaddr_un sun;
	struct sockaddr_in sin;
} TTMPSockAddr;

/* in_addr_t: IPv4 address type */
/* in_addr_t defined in netinet/in.h */

/* Socket and socket error magic values */
#define INVALID_DOMAIN (-1)
#define INVALID_SOCKET (-1)
#define SOCKET_ERROR   (-1)

/* Socket function aliases */
#define closesocket   close
#define ioctlsocket   ioctl
#define socket_errno  errno

/* Socket macros */
#define TTMPSocketUnlink(sa)										\
	do {															\
		if ((sa)->domain == AF_UNIX)								\
			unlink((sa)->addr.sun.sun_path); /* Ignore errors */	\
	} while (0)

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* INCLUDED_TTMPPLAT_H */

/* EOF */
